
//----------------------------------------------------------------------//
class Company
{
   private String Name;
   private String Website;

   private Company next;
   private Company previous;

   Company (String Name, String Website)
   {
     this.Name = Name;   //set company name
     this.Website = Website; //set company website

     this.setNext(null);        //next node is null
     this.setPrevious(null);    //previous node is null
   }


   public void setData(String Name, String Website)
   {
     this.Name = Name; //set new company name
     this.Website = Website; //set new company website
   }

   public String getName()
   {
      return this.Name;
   }

   public String getWebsite()
   {
      return this.Website;
   }

   public void setNext(Company next)
   {
     this.next = next;  //set next company
   }

   public Company getNext()
   {
     return this.next;  //return next company in list
   }

   public void setPrevious(Company previous)
   {
     this.previous = previous;  //set previous company
   }

   public Company getPrevious()
   {
     return this.previous;  //return previous company
   }

}  //end of Company class definition

//----------------------------------------------------------------------//
class CompanyList
{
//company list is a COLLECTION of Company objects
// linked list style: each node(company) has references to the next and
// previous companies in the list

   private Company top;
   private Company bottom;
   private int CompanyCount;

   private Company CurrentCompany;
   private int CurrentIDX;         //NOTE: OPTION BASE 1!!!

   CompanyList()
   {
     this.CompanyCount = 0;  //no companies added yet
     this.CurrentIDX = 1;
     this.top = null;  //no top company
     this.bottom = null; //no bottom company yet
   }

   public void add(String CompanyName, String CompanyWebsite)
   {
     if (this.CompanyCount == 0)  //first company in list?
     {
        top = new Company(CompanyName, CompanyWebsite); //define top company
        CurrentCompany = top;
        this.CompanyCount++; //company added, increment count
     }
     else  //atleast one company in list:
     {
       Company cp = new Company(CompanyName, CompanyWebsite); //create new company
       cp.setNext(this.top);  //set the next node to top (inserts to beginning of list)
       this.top.setPrevious(cp);  //old top's previous is now new top
       this.top = cp;         //this node is now top, because it was inserted in beginning
       this.CompanyCount++; //company added, increment count
       CurrentCompany = top;
     }
   }

   public void increment()  //set current company to next in list
   {
     if ( CompanyCount <= 0 ) return;

     CurrentIDX++; //increment to next company

     if ( CurrentIDX >= CompanyCount )
       CurrentIDX = CompanyCount;

     if ( CurrentIDX == 1 )  //current index is first company?
       CurrentCompany = top;
     else
       CurrentCompany = CurrentCompany.getNext(); //get next company
   }   

   public void decrement()
   {
     if ( CompanyCount <= 2 ) return;

     CurrentIDX--;
     
     if ( CurrentIDX <= 1 ) CurrentIDX = 2;  //must be atleast 2

     CurrentCompany = CurrentCompany.getPrevious();
   }

   public Company getCompany()   //get current company
   {
     return CurrentCompany;
   }

   public int getCompanyCount()
   {
     return this.CompanyCount;
   }




} //end of CompanyList class definition
//----------------------------------------------------------------------//
public class ListTest
{

  public static void main (String [] args)
  {
    CompanyList cpl = new CompanyList();
    Company cy;

    cpl.add("Nike","www.nike.com");              //add some companies...
    cpl.add("Microsoft", "www.microsoft.com");
    cpl.add("XDE", "www.xde.net");
    cpl.add("Amazon", "www.amazon.com");
    cpl.add("Sony", "www.sony.com");

    System.out.println("Company Count: " + cpl.getCompanyCount());
    System.out.println("---------------------------------------------------------");

  //enumeration style through list: (i's value is NOT used, but rather the for loop
  //    is used because there will be a set number of iterations, company count)

    for (int i=1; i <= cpl.getCompanyCount(); i++)
    {
      cy = cpl.getCompany();  //get current company (at start, current company is first one, which is last one added)
      System.out.println( cy.getName() + "\t\t\t" + cy.getWebsite() );  //print company info

      cpl.increment();   //set current company to next in list
    }
    System.out.println("---------------------------------------------------------");

  }



}
